@@ -4,8 +4,15 @@
 #ifndef BUILDINFO_H
 #define BUILDINFO_H
 
+    #undef ERROR
     #include <R.h>
 
+    // for windows and Mac build need to undo declarations in R.h (R>1.8.0)
+    #ifdef IEEE_754
+    #undef ISNAN
+    #define ISNAN(x)   R_IsNaNorNA(x)
+    #endif
+
     #define GBM_FAILED(hr) ((unsigned long)hr != 0)
     typedef unsigned long GBMRESULT;
     #define GBM_OK 0
@@ -24,4 +31,3 @@
 
 #endif // BUILDINFO_H
 
-
